#ifndef BEAMERLOCATIONGUI_H
#define BEAMERLOCATIONGUI_H

#include <QDialog>
#include <QTimer>
#include <QMutex>
#include <opencv2/opencv.hpp>
#include "qtfullscreen.h"
#include "monitorgui.h"
#include "sandboxSetup.h"
#include "subapp.h"

namespace Ui {
class BeamerLocationGui;
}

class BeamerLocationGui : public SubApp
{
    Q_OBJECT

public:
    explicit BeamerLocationGui(SandboxSetup *_setup, MonitorGui *_mg, QWidget *parent = 0);
    ~BeamerLocationGui();
    void valideRoutine();
    void cancelRoutine();
    void routineFrame();

protected:
    void closeEvent (QCloseEvent *event);
    void showEvent(QShowEvent *event);

private slots:


    void on_btnLock_clicked();

    void on_btnRestart_clicked();

private:
    class RefreshFrame : public QThread
    {
        public:
            explicit RefreshFrame(BeamerLocationGui *routine);
            void run();

        private:
            BeamerLocationGui *routine;
    };

    Ui::BeamerLocationGui *ui;
    QtFullScreen *winFullScreen;
    SandboxSetup *setup;
    Beamer *beamer;
    Camera *camera;
    FrameProcessProfil *profil;
    MonitorGui *mg;
    int keypressed = -1;
    bool isFullScreenInit = false;
    bool isWaiting = true;
    std::vector<cv::Point2i> crosses;
    std::vector<cv::Point3d> directions;
    std::vector<cv::Point3d> bases;
    std::vector<cv::Point3f> capturedPoints;
    cv::Mat depth;
    cv::Mat rgb;
    std::vector<cv::Point3i> circle;
    int stepCross = 0;
    QTimer *timer;
    RefreshFrame *myThread;

    void startRoutine();
    void userValidePoint();
    void endRoutine();
    void refreshRoutine();
    void updateLabelSteps();
};

#endif // BEAMERLOCATIONGUI_H
