#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTimer>

#include <sandboxSetup.h>
#include "subapp.h"

#include "monitorgui.h"
#include "camerafocus.h"
#include "croppingmask.h"
#include "projectiongui.h"
#include "qtfullscreen.h"
#include "beamerlocationgui.h"
#include "initcamera.h"
#include "saveconfiggui.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private slots:
    void on_btnNext_clicked();

    void on_btnPrev_clicked();

    void receiveNotif(int state);

    void on_btnFinish_clicked();

protected:
    void showEvent(QShowEvent *event);
    void closeEvent(QCloseEvent *event);

private:
    Ui::MainWindow *ui;
    SandboxSetup *setup;
    InitCamera *init;
    MonitorGui *mg;
    ProjectionGui *pg;
    CameraFocus *cf;
    CroppingMask *cm;
    BeamerLocationGui *bl;
    SaveConfigGui *scfg;
    int step = 0;
    std::vector<SubApp*> applist;

    void loadApp();
    void closeApp();
    bool verifyApp();
    void nextApp();
    void prevApp();

    void displayButtons();
    void exit_msg(std::string msg);
};

#endif // MAINWINDOW_H
