#ifndef BEAMER_H
#define BEAMER_H
#include <opencv2/opencv.hpp>
#include "camera.h"


class FrameProcessProfil{
    private:
        // Profil to process on gray scaled images
        double contrast; // ]0, 255]
        int brightness; // [-2^16, 255]
        uint minDistance; // [1, 100] % of the frame's width
        uint cannyEdgeThreshold; // [0, 255] the strongest the contrast in the image is, the higher this param should be
        uint houghAccThreshold; // [0, n] where n = width*height of the frame
        uint minRadius; // [1, 100] % of the frame's width
        uint maxRadius; // [1, 100] % of the frame's width

    public:
        FrameProcessProfil(double c=1.0, int b=0, int d=15, uint cannyThreshold=100, uint accThreshold=50, uint minR=10, uint maxR=50){
            contrast = c;
            brightness = b;
            minDistance = d;
            cannyEdgeThreshold = cannyThreshold;
            houghAccThreshold = accThreshold;
            minRadius = minR;
            maxRadius = maxR;
        };
        double getContrast(){ return contrast; };
        void setContrast(double c){ contrast = c; };
        int getBrightness(){ return brightness; };
        void setBrightness(int b){ brightness = b; };
        uint getMinDistance(){ return minDistance; };
        void setMinDistance(uint d){ minDistance = d; };
        uint getCannyEdgeThreshold(){ return cannyEdgeThreshold; };
        void setCannyEdgeThreshold(uint cannyThreshold){ cannyEdgeThreshold = cannyThreshold; };
        uint getHoughAccThreshold(){ return houghAccThreshold; };
        void setHoughAccThreshold(uint accThreshold){ houghAccThreshold = accThreshold; };
        uint getMinRadius(){ return minRadius; };
        void setMinRadius(uint min){ minRadius = min; };
        uint getMaxRadius(){ return maxRadius; };
        void setMaxRadius(uint max){ maxRadius = max; };

        void print(){
            std::cout << contrast << std::endl;
            std::cout << brightness << std::endl;
            std::cout << minDistance << std::endl;
            std::cout << cannyEdgeThreshold << std::endl;
            std::cout << houghAccThreshold << std::endl;
            std::cout << minRadius << std::endl;
            std::cout << maxRadius << std::endl;
        }

};

class Beamer{
    private:
        cv::Point3f beamerPosition;
        cv::Size resolution = cv::Size(256,144);
        FrameProcessProfil profil = FrameProcessProfil();

        cv::Point3f intersection(cv::Vec3f v1, cv::Point3f p1, cv::Vec3f v2, cv::Point3f p2, cv::Vec3f v3, cv::Point3f p3, bool &isFound);
        
    public:
        Beamer();

        const char ESCAPE_CHAR = 27;
        const unsigned int MAX_LINEAR_LINE_POINTS = 3; //number of point to calculate 1 vector
        const double LINEAR_REGRESSION_FACT = -20.0; // used for linear regression

        cv::Point3f getPosition(){ return beamerPosition; };
        void setPosition(cv::Point3f pos){ beamerPosition = pos; };
        
        void setWidth(int w){ resolution.width = w; };
        int getWidth(){ return resolution.width; };
        void setHeight(int h){ resolution.height = h; };
        int getHeight(){ return resolution.height; };
        void setResolution(cv::Size res){ 
            setWidth(res.width);
            setHeight(res.height);
        }

        FrameProcessProfil* getProfil(){ return &profil; };
        void setProfil(FrameProcessProfil p){ profil = p; };

        int findBeamerFrom(Camera *camera);
        cv::Mat editContrast(cv::Mat image, double contrast, int brightness);
        cv::Point3f deprojectPixel(cv::Point2i circle, cv::Mat *depth, Camera *camera);
        std::vector<cv::Point2i> getCrossList();
        cv::Mat getCrossFrame(cv::Point2i projectedCross, int step, int max, bool circlesFound);
        cv::Point3d approximatePosition(std::vector<cv::Point3d> *bases, std::vector<cv::Point3d> *directions);
        void findLinearLineFrom(std::vector<cv::Point3f> *capturedPoints, std::vector<cv::Point3d> *bases, std::vector<cv::Point3d> *directions);
        std::vector<cv::Point3i> findCircles(cv::Mat &rgb, double contrast, int brightness, double centersMinDist, int cannyEdgeThreshold, int houghAccThreshold, double minRadius, double maxRadius);
        void printPosition();
        
};
#endif
