#ifndef CAMERA_H
#define CAMERA_H

#include <librealsense2/rs.hpp>
#include <librealsense2/rsutil.h>
#include <opencv2/opencv.hpp>

class Camera{
    private:
        rs2::spatial_filter *spatFilter;
        rs2::temporal_filter *tempFilter;
        rs2::config *cfg;
        rs2::pipeline *pipe;
        rs2::align *align_to_depth;

        rs2::video_frame *color_frame;
        rs2::depth_frame *depth_frame;
    
        cv::Rect croppingMask;

        void warmUpDepthLens();
        void filterDepthFrame(rs2::depth_frame &frameDepth);

    public:
        Camera();
        ~Camera();

        // return a float matrix of depth in meters
        cv::Mat getDepthFrame(){
            cv::Mat meters;
            cv::Mat values = cv::Mat(cv::Size(depth_frame->get_width(), depth_frame->get_height()), CV_16UC1, (void *)depth_frame->get_data(), cv::Mat::AUTO_STEP);
            values.convertTo(meters,CV_32FC1, depth_frame->get_units());
            return meters;
        };
        cv::Mat getColorFrame(){ return cv::Mat(cv::Size(color_frame->get_width(), color_frame->get_height()), CV_8UC3, (void *)color_frame->get_data(), cv::Mat::AUTO_STEP); };
        void setCroppingMask(cv::Rect mask){ croppingMask = mask; };
        cv::Rect getCroppingMask(){ return croppingMask; };
        
        int start();
        void stop();
        cv::Point3f deprojectPixelToPoint(float coord[], float z1);
        cv::Point2i projectPointToPixel(cv::Point3f point3D);
        void capture();
        void printCroppingMask();

};
#endif