#ifndef SANDBOXSETUP_H
#define SANDBOXSETUP_H

#include <opencv2/opencv.hpp>
#include "beamer.h"
#include "beamerProjection.h"
#include "camera.h"
#include "borderedit.h"
#include "sandboxConfig.h"

class SandboxSetup{
    private:
        char *defaultConfigFilePath = (char *)"./sandbox_conf.yaml";
        BeamerProjection projection;
        Camera camera;
        Beamer beamer;

        double toDegrees(double radians);
        void initWindowsFullScreen(char *windowName);

    public:
        SandboxSetup();

        BeamerProjection* getProjection(){ return &projection; };
        Camera* getCamera(){ return &camera; };
        Beamer* getBeamer(){ return &beamer; };
        
        
        // save config in file => persistant
        int saveConfigFrom(char *path);
        int saveConfig();
        
        int loadFrameProcessProfil();

        // edit variables of config => not persistant
        void setupAdjustMatrix(std::vector<cv::Point> rectPoints, cv::Point center);
        void setupCroppingMask(std::vector<cv::Point> rectPoints);
        
        void defaultCaptureBlueScreen(int delay);
        int defaultSetupProjection();
        int defaultSetupBeamerResolution();
        int defaultSetupBeamerLocation();

};

#endif
