#include "../../inc/beamerProjection.h"


BeamerProjection::BeamerProjection(){
    //adjustingMatrix = cv::getRotationMatrix2D(cv::Point(0, 0), 0, 1);
}


cv::Point2i BeamerProjection::rotatePixel(cv::Point2i pixel){

    cv::Mat tmp = (cv::Mat_<cv::Vec2f>(1, 1) << cv::Vec2f(pixel.x, pixel.y));
    cv::transform(tmp, tmp, adjustingMatrix);
    return cv::Point2i(tmp.at<cv::Vec2f>(0, 0));
}



void BeamerProjection::adjustFrame(cv::Mat depth, cv::Mat src, cv::Mat &dst, Camera *camera, cv::Point3f beamer_pos){

    cv::Rect mask = camera->getCroppingMask();
    cv::Size dst_size = dst.size();

    // resize the frames to be a multiple of the camera size :
    //      src.size = n * camera.depth.size , where n is uint > 0
    cv::resize(dst, dst, getMatchingSize(dst, mask));
    cv::resize(src, src, dst.size());
   
    cv::Mat pixelsDeprojectMap = cv::Mat_<cv::Point2i>(mask.height, mask.width, cv::Point2i(-1,-1));
    cv::Mat pixelsDeprojectHighestMap = cv::Mat_<cv::Point2i>(mask.height, mask.width, cv::Point2i(-1,-1));

    // Browse the depth frame matching the cropping mask
    // while adapting pixels's position to the beamer's position
    for (int j = 0; j < depth.rows; j++){
        for (int i = 0; i < depth.cols; i++){

            // coordinates of the pixel relative to the orginial image taken from the camera
            int base_x = mask.x+i;
            int base_y = mask.y+j;
            float z = depth.at<float>(j,i);
            
            // pixels based on the original depth frame taken from the camera
            cv::Point2i pixel = findMatchingPixel( base_x, base_y, z, camera, beamer_pos );
            
            // pixel relative to the cropping mask (the area where the frame is projected)
            pixel.x -= mask.x;
            pixel.y -= mask.y;

            pixelsDeprojectMap.at<cv::Point2i>(j,i) = pixel;
        }
    }

    for (int j = 0; j < pixelsDeprojectMap.rows; j++){
        for (int i = 0; i < pixelsDeprojectMap.cols; i++){

            cv::Point2i pixel = pixelsDeprojectMap.at<cv::Point2i>(j,i);

            if(pixel.x != -1 && pixel.y != -1){
                // check and keep the highest point at the location pointed by pixel
                cv::Point2i defaultPoint = pixelsDeprojectHighestMap.at<cv::Point2i>(j,i);
                if(defaultPoint.x != -1 && defaultPoint.y != -1){
                    if(depth.at<float>(defaultPoint) <= depth.at<float>(pixel))
                        pixel = defaultPoint;
                }
                pixelsDeprojectHighestMap.at<cv::Point2i>(j,i) = pixel;
            }
        }
    }


    for (int j = 0; j < pixelsDeprojectHighestMap.rows; j++){
        for (int i = 0; i < pixelsDeprojectHighestMap.cols; i++){

            cv::Point2i pixel = pixelsDeprojectHighestMap.at<cv::Point2i>(j,i);

            if( (0<=pixel.x && pixel.x<depth.cols) && (0<=pixel.y && pixel.y<depth.rows) ){
                // src and dst must be of same size
                copyPixelsInto(pixel, dst, cv::Point2i(i,j), src, mask);
            }
        }
    }
  

    cv::resize(dst, dst, dst_size);
    cv::warpAffine(dst, dst, adjustingMatrix, dst.size());
}

cv::Size BeamerProjection::getMatchingSize(cv::Mat &src, cv::Rect base){
    cv::Size bigSize;
    bigSize.width = (src.size().width % base.width == 0) ? src.size().width : src.size().width - (src.size().width % base.width) + base.width;
    bigSize.height = (src.size().height % base.height == 0) ? src.size().height : src.size().height - (src.size().height % base.height) + base.height;
    return bigSize;
}

// pixels coordinates are relative to the camera depth frame
void BeamerProjection::copyPixelsInto(cv::Point2i pixel_dst, cv::Mat &dst, cv::Point2i pixel_src, cv::Mat &src, cv::Rect base){

    if( src.size().width == dst.size().width && src.size().height == dst.size().height ){

        // determines the size of the square of pixels to copy from the source matching the position of the pixel of the depth frame
        // (since src and dst are multiple of the depth frame's size)
        int ratio_w = src.size().width / base.width;
        int ratio_h = src.size().height / base.height;

        for(int j=0; j<ratio_h; j++){
            for (int i=0; i<ratio_w; i++){

                int x_dst = pixel_dst.x * ratio_w + i;
                int y_dst = pixel_dst.y * ratio_h + j;
                int x_src = pixel_src.x * ratio_w + i;
                int y_src = pixel_src.y * ratio_h + j;

                dst.at<cv::Vec3b>(y_dst, x_dst) = src.at<cv::Vec3b>(y_src, x_src);
            }
        }
    }
}

/*
    C : Camera position
    B : Beamer position
    P : Point computed by camera depth
    
    Where
        CP : distance from camera to point (value of depth_frame)
        CB : distance from camera to beamer (beamer's position is relative to the camera)
*/
cv::Point2i BeamerProjection::findMatchingPixel(int i, int j, float z, Camera *camera, cv::Point3f CB){
   
    float pixel[2] = {static_cast<float>(i), static_cast<float>(j)};
    cv::Point3f CP = camera->deprojectPixelToPoint(pixel, z);
    cv::Point3f BP = CB - CP;
    float BAz = CB.z + BP.z;
    float BEz = distanceTopSandbox - CB.z;
    float alpha = BEz / BAz;
    cv::Point3f V = (alpha * BP);
    cv::Point3f CV = V + CB;

    return camera->projectPointToPixel(CV);
}


void BeamerProjection::printAdjustingMatrix(){

    cv::Mat matrix = getAdjustingMatrix();
    for (int y = 0; y < matrix.rows; y++){
        for (int x = 0; x < matrix.cols; x++){
            std::cout << (float)matrix.at<double>(y, x) << " ";
        }
        std::cout << std::endl;
    }
}