#include "../../inc/sandbox.h"


/*
 *   MAIN
 */

Sandbox::Sandbox(){
    
}


/*
 *   PUBLIC
 */

void Sandbox::init(){
    camera.start();
}

cv::Mat Sandbox::getColorFrame(){
    camera.capture();
    return camera.getColorFrame()(camera.getCroppingMask());
}

cv::Mat Sandbox::getDepthFrame(){
    camera.capture();
    return camera.getDepthFrame()(camera.getCroppingMask());
}


cv::Mat Sandbox::adjustProjection(cv::Mat frame){
    
    cv::Mat depth = getDepthFrame();
    cv::Mat imageCalibrate = cv::Mat(cv::Size(beamer.getWidth(), beamer.getHeight()), CV_8UC3, cv::Scalar(0, 0, 0));
    projection.adjustFrame(depth, frame, imageCalibrate, &camera, beamer.getPosition());

    // frame after process
    //cv::dilate(imageCalibrate, imageCalibrate, cv::Mat(), cv::Point(-1, -1), 2, 1, 1);
    //cv::erode(imageCalibrate, imageCalibrate, cv::Mat(), cv::Point(-1, -1), 2, 1, 1);
    
    return imageCalibrate;
}

void Sandbox::showImage(cv::Mat &image){

    initWindowsFullScreen(defaultWindowsName);
    cv::Mat res = adjustProjection(image);
    cv::imshow(defaultWindowsName, res);
}

int Sandbox::loadConfigFrom(char *path){

    int err = SandboxConfig::loadAdjustingMatrixFrom(path, &projection);
    if(err){ return err; }
    err = SandboxConfig::loadDistanceToSandboxTop(path, &projection);
    if(err){ return err; }
    err = SandboxConfig::loadCroppingMaskFrom(path, &camera);
    if(err){ return err; }
    err = SandboxConfig::loadBeamerResolutionFrom(path, &beamer);
    if(err){ return err; }
    err = SandboxConfig::loadBeamerPositionFrom(path, &beamer);
    if(err){ return err; }
    return 0;
}

int Sandbox::loadConfig(){

    return loadConfigFrom(defaultConfigFilePath);
}

void Sandbox::initWindowsFullScreen(){
    initWindowsFullScreen(defaultWindowsName);
}


/*
 *   PRIVATE
 */

void Sandbox::initWindowsFullScreen(char *windowName){
    cv::namedWindow(windowName, CV_WINDOW_NORMAL);
    cv::setWindowProperty(windowName, CV_WND_PROP_FULLSCREEN, CV_WINDOW_FULLSCREEN);
}