#include "../../inc/sandboxSetup.h"


SandboxSetup::SandboxSetup(){
    
}

//
//  PUBLIC
//


// return 1 when config can't be saved in file
int SandboxSetup::saveConfigFrom(char *path){
    if(SandboxConfig::saveAdjustingMatrixInto(path, projection.getAdjustingMatrix()))
        return 1;
    
    if(SandboxConfig::saveDistanceToSandboxTop(path, projection.getDistanceTopSandbox()))
        return 1;

    if(SandboxConfig::saveCroppingMaskInto(path, camera.getCroppingMask()))
        return 1;

    if(SandboxConfig::saveBeamerResolutionInto(path, cv::Size(beamer.getWidth(), beamer.getHeight())))
        return 1;

    if(SandboxConfig::saveBeamerPositionInto(path, beamer.getPosition()))
        return 1;

    if(SandboxConfig::saveFrameProcessProfil(path, *beamer.getProfil()))
        return 1;

    return 0;
}


int SandboxSetup::saveConfig(){
    return saveConfigFrom(defaultConfigFilePath);
}

int SandboxSetup::loadFrameProcessProfil(){
    return SandboxConfig::loadFrameProcessProfil(defaultConfigFilePath, beamer.getProfil());
}


void SandboxSetup::setupAdjustMatrix(std::vector<cv::Point> rectPoints, cv::Point center){

    // Set adjusting matrix for the projection
    int widthTop = rectPoints[3].x - rectPoints[0].x;
    double angle1 = atan((double)(rectPoints[3].y - rectPoints[0].y) / widthTop);
    cv::Mat matRotation = cv::getRotationMatrix2D(center, toDegrees(angle1), 1); // adjustingMatrix
    projection.setAdjustingMatrix(matRotation);
}

void SandboxSetup::setupCroppingMask(std::vector<cv::Point> rectPoints){

    // Set cropping mask
    int widthTop = rectPoints[3].x - rectPoints[0].x;
    cv::Size rectSize = cv::Size(widthTop, cvRound(widthTop / 1.33333) + 5);
    cv::Point p = projection.rotatePixel(rectPoints[0]);
    camera.setCroppingMask(cv::Rect(p, rectSize)); // croppingMask
}



//
// Default setup routines
//

int SandboxSetup::defaultSetupBeamerResolution(){
    int width = 0;
    int height = 0;

    std::cout << "Enter width of the frame projected :" << std::endl;
    std::cin >> width;
    if(std::cin.fail()){
        return 1;
    }
    std::cout << "Enter height of the frame projected :" << std::endl;
    std::cin >> height;
    if(std::cin.fail()){
        return 1;
    }

    beamer.setHeight(height);
    beamer.setWidth(width);

    return 0;
}


// return 1 when user exits process
int SandboxSetup::defaultSetupBeamerLocation(){

    return beamer.findBeamerFrom(&camera);
}


// return 1 when user exits process
int SandboxSetup::defaultSetupProjection(){

    defaultCaptureBlueScreen(300);

    cv::Mat frameData = camera.getDepthFrame();
    cv::Mat coloredFrame = camera.getColorFrame();
    cv::Size s = frameData.size();
    cv::Point center(s.width / 2, s.height / 2);

    // Edit projection
    float y = coloredFrame.size().height;
    float x = coloredFrame.size().width;
    std::vector<cv::Point> rectPoints{ cv::Point(1.0/4*x, 1.0/4*y), cv::Point(1.0/4*x, 3.0/4*y), cv::Point(3.0/4*x, 3.0/4*y), cv::Point(3.0/4*x, 1.0/4*y) };
    if(BorderEdit::edit(coloredFrame, &rectPoints)){
        return 1;
    }
    cv::destroyAllWindows();

    
    // must be called before setupCroppingMask
    setupAdjustMatrix(rectPoints, center);

    setupCroppingMask(rectPoints);

    return 0;
}


void SandboxSetup::defaultCaptureBlueScreen(int delay){
    
    // Blue screen 
    char windowName[] = "BlueScreen";
    initWindowsFullScreen(windowName);
    cv::Mat frameBeamer(cv::Size(beamer.getWidth(), beamer.getHeight()), CV_8UC3, cv::Scalar(255, 0, 0));
    cv::imshow(windowName, frameBeamer);
    delay = (delay==0)? 1 : delay;
    cv::waitKey(delay);

    // Take picture
    camera.capture();
    cv::destroyAllWindows();
}



//
//  PRIVATE
//


double SandboxSetup::toDegrees(double radians){
    return radians * (180.0 / M_PI);
}


void SandboxSetup::initWindowsFullScreen(char *windowName){
    cv::namedWindow(windowName, CV_WINDOW_NORMAL);
    cv::setWindowProperty(windowName, CV_WND_PROP_FULLSCREEN, CV_WINDOW_FULLSCREEN);
}
