#include "../../inc/borderedit.h"


cv::Mat BorderEdit::frameImage;


int BorderEdit::edit(cv::Mat frame, std::vector<cv::Point> *posSandbox){

    cv::namedWindow(wndname, CV_WINDOW_AUTOSIZE);
    frame.copyTo(frameImage);
    cv::setMouseCallback(wndname, BorderEdit::mouseHandler, (void *)posSandbox);
    BorderEdit::drawSquare(&posSandbox->at(0), (int)posSandbox->size());
    char keyCode = cv::waitKey(0);
    if (keyCode == ESCAPE_CHAR){
        return 1;
    }
    return 0;
}


void BorderEdit::mouseHandler(int event, int x, int y, int, void *param){

    static int selectedPoint = -1;
    std::vector<cv::Point> &posSandbox = *((std::vector<cv::Point> *)param);
    switch (event)
    {
    case CV_EVENT_LBUTTONDOWN: //left button press
        selectedPoint = BorderEdit::findPoints(x, y, posSandbox);
        break;

    case CV_EVENT_LBUTTONUP: //left mouse button release
        if (selectedPoint != -1)
            selectedPoint = -1;
        break;

    case CV_EVENT_MOUSEMOVE:
        /* draw a rectangle*/
        if (selectedPoint != -1)
        {
            posSandbox[selectedPoint].x = x;
            posSandbox[selectedPoint].y = y;
            BorderEdit::drawSquare(&posSandbox[0], (int)posSandbox.size());
            //Rect r = boundingRect(posSandbox);
            //double f = (double)r.width / (double)r.height;
            //cout << 4.0 / 3.0 << " == " << f << endl;
        }
        break;
    }
}


void BorderEdit::drawSquare(cv::Point *p, int n){

    cv::Mat imageCopy = frameImage.clone();
    polylines(imageCopy, &p, &n, 1, true, cv::Scalar(0, 255, 0), 1, cv::LINE_AA);
    imshow(wndname, imageCopy);
}


int BorderEdit::findPoints(int x, int y, std::vector<cv::Point> &posSandbox){

    for (int i = 0; i < (int)posSandbox.size(); i++)
    {
        //check if the point is clicked
        if ((x >= (posSandbox[i].x - margeClick)) && (x <= (posSandbox[i].x + margeClick)) && (y >= (posSandbox[i].y - margeClick)) && (y <= (posSandbox[i].y + margeClick)))
            return i;
    }
    return -1; //not found
}

